//
// WINDOW.HPP   Interface for Window class.
//
// Created:     Bruce Clift  6/25/91
//

#ifndef __WINDOW_HPP
#define __WINDOW_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __LIST_HPP
#include "list.hpp"
#endif

#ifndef __WINDEF_H
#include "windef.h"
#endif

class RegWinCls;
class Handler;
class Ribbon;
class StatusBar;


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Window {

    ////////////////////////
    // INSTANCE VARIABLES //
    ////////////////////////

    // Window handle list variables
public:
    static Window   *pNewWin;   // New object to add to window handle list
private:
    static List     hwndList;   // List of created window handles

    // Window information
    HWND            _hw;        // Window handle of this object
    char            *pCapt;     // Window caption title
    LPCSTR          pMenu;      // Menu resource name
protected:
    Handler         *_pHand;    // Pointer to message Handler object

    // Bit-field flags
    struct windowFlags {
        unsigned    created : 1;    // Was this window created?
        unsigned    autoHand : 1;   // Using an auto-created Handler?
        unsigned    fail : 1;       // Fail window creation?
        windowFlags() : created(FALSE), autoHand(FALSE), fail(FALSE) {}
    } f;


public:
    ////////////////////////////////////////////////
    // WINDOW OBJECT CONSTRUCTION AND DESTRUCTION //
    ////////////////////////////////////////////////

    // Default constructor simply clears instance variables
    Window();

    // Other constructors get the handle of an EXISTING window
    Window(LPCSTR lpClass, LPCSTR lpWindow = NULL)
        : _hw(NULL), pCapt(NULL), pMenu(NULL), _pHand(NULL)
        { find(lpClass, lpWindow); }    // Find an existing window
    Window(POINT p)
        : _hw(NULL), pCapt(NULL), pMenu(NULL), _pHand(NULL)
        { fromPoint(p); }               // Find at location
    Window(HWND hw)
        : _hw(NULL), pCapt(NULL), pMenu(NULL), _pHand(NULL)
        { get(hw); }                    // Make object for this window
    Window(HWND hw, UINT wCmd)
        : _hw(NULL), pCapt(NULL), pMenu(NULL), _pHand(NULL)
        { get(hw, wCmd); }              // Get from relationship

    // Methods to support above constructors
    // (and other initialization methods which set the window handle)
    HWND find(LPCSTR lpClass, LPCSTR lpWindow = NULL)
        { return (_hw=FindWindow(lpClass, lpWindow)); }
    HWND childFromPoint(HWND hw, POINT p)
        { return (_hw=ChildWindowFromPoint(hw, p)); }
    HWND fromPoint(POINT p)         { return (_hw=WindowFromPoint(p)); }
    HWND get(HWND hw)               { return (_hw=hw); }
    HWND get(HWND hw, UINT wCmd)    { return (_hw=GetWindow(hw, wCmd)); }
    HWND getParent(HWND hw)         { return (_hw=GetParent(hw)); }
    HWND getDesktop(void)           { return (_hw=GetDesktopWindow()); }
    HWND getActive(void)            { return (_hw=GetActiveWindow()); }
    HWND getTop(HWND hw)            { return (_hw=GetTopWindow(hw)); }
    HWND getNext(HWND hw, UINT wFlag) { return (_hw=GetNextWindow(hw, wFlag)); }

    // Destructor clears window handle member (& removes from list)
    virtual ~Window();
    virtual BOOL destroy(void);

    // Method to create a Window object
    HWND create(LPCSTR cls, LPCSTR name, HINSTANCE hInst,
        long    lStyle      = DEF_SETSTYLE,
        HMENU   hMenu       = DEF_HMENU,
        HWND    hwParent    = DEF_HWPARENT,
        int     x           = DEF_X,
        int     y           = DEF_Y,
        int     cx          = DEF_CX,
        int     cy          = DEF_CY,
        LPSTR   lpParam     = DEF_LPPARAM,
        DWORD   dwExStyle   = DEF_EXSTYLE
    );
    virtual HWND create(LPCSTR name, HINSTANCE hInst,
        long    lStyle      = DEF_SETSTYLE,
        HMENU   hMenu       = DEF_HMENU,
        HWND    hwParent    = DEF_HWPARENT,
        int     x           = DEF_X,
        int     y           = DEF_Y,
        int     cx          = DEF_CX,
        int     cy          = DEF_CY,
        LPSTR   lpParam     = DEF_LPPARAM,
        DWORD   dwExStyle   = DEF_EXSTYLE
    );
    virtual LPCSTR getClassName(void);

    // Other Window initialization methods
    virtual BOOL regClass(HINSTANCE hInst, char *szName);
    virtual void setClassReg(RegWinCls &);
    virtual BOOL regChildClass(HINSTANCE hInst);
    virtual BOOL display(HINSTANCE hInst, char *szApp, char *szCapt=NULL,
        int nCmdShow = SW_SHOWNORMAL);
    virtual long getStyle(void);


    ////////////////////////////////////////////////////////////
    // MESSAGE PROCESSING WITH THE HANDLER OBJECT (WINCB.CPP) //
    ////////////////////////////////////////////////////////////

    // Get/set a pointer to the Handler for this Window
    virtual Handler *getHandler(void);
    void setHandler(Handler *pHand) { _pHand = pHand; }

    // Default message callback registration
    virtual void regDefMsgs(Handler *pHand);
    LRESULT wmDestroy(UINT, WPARAM, LPARAM);

    // Dispatch a message to the proper method
    virtual LRESULT dispatch(UINT wMsg, WPARAM wParam, LPARAM lParam);

    // Call default window procedure
    virtual LRESULT callDefProc(UINT wMsg, WPARAM wParam, LPARAM lParam);
    virtual LRESULT callDefProc(FARPROC defWinProc, UINT, WPARAM, LPARAM);


    ////////////////////////////
    // WINDOW CLASS UTILITIES //
    ////////////////////////////

    // Member access functions
    HWND hwnd(HWND hw)  { return (_hw = hw); }
    HWND hwnd(void)     { return _hw; }

    void setMenuName(LPCSTR pMenuName)  { pMenu = pMenuName; }
    void setMenuName(int iRes)          { pMenu = MAKEINTRESOURCE(iRes); }
    LPCSTR getMenuName(void)            { return pMenu; }

    void setCaption(char *pCaption)     { pCapt = pCaption; }
    char *getCaption(void)              { return pCapt; }

    void fail(BOOL bFail) { f.fail = bFail; }

    // Overloaded operators
    BOOL operator! (void) { return (_hw == NULL); }
    operator HWND (void) { return (_hw); }

    // Window handle list management methods
    static Window *newWinHnd(HWND hw);
    static Window *findWinHnd(HWND hw);
    Window  *addWinHnd(HWND hw);
    BOOL    destroyWinHnd(void) { return hwndList.remove(_hw); }


    //////////////////////////////////
    // SUPPLEMENTARY OBJECT SUPPORT //
    //////////////////////////////////

    // Provide pointers to supplementary objects
    virtual Ribbon *    getRibbon(void);
    virtual StatusBar * getStatusBar(void);
    virtual HACCEL      getAccel(void);
    virtual Window *    getActiveChild(BOOL *pbMax = NULL);

    // Override this method to initialize a Status Bar, Ribbon, etc.
    virtual BOOL setChildren(void);

    // Message callbacks for status bars only
    LRESULT wmPaint(UINT, WPARAM, LPARAM);
    LRESULT wmMenuSelect(UINT, WPARAM, LPARAM);
    LRESULT wmEnterIdle(UINT, WPARAM, LPARAM);

    // Methods to provide status bar help index values
    virtual int getHelpIndex(void);         // Non-menu help
    virtual int getMenuHelpIndex(int);      // Menu item help
    virtual int getSysMenuHelpIndex(int);   // System menu item help
    virtual int getPopupHelpIndex(int);     // Menu popup help

    // Convert the saved status bar help index into a help string
    virtual char *getHelp(int);
    virtual char *getHelp(void);


    /////////////////////////////
    // CALLING THE WINDOWS API //
    /////////////////////////////

    // Get/set class/window word/long extra bytes
    WORD    getClassWord(int n)         { return GetClassWord(_hw, n); }
    LONG    getClassLong(int n)         { return GetClassLong(_hw, n); }
    WORD    setClassWord(int n, WORD w) { return SetClassWord(_hw, n, w); }
    LONG    setClassLong(int n, LONG l) { return SetClassLong(_hw, n, l); }
    WORD    getWord(int n)              { return GetWindowWord(_hw, n); }
    LONG    getLong(int n)              { return GetWindowLong(_hw, n); }
    WORD    setWord(int n, WORD w)      { return SetWindowWord(_hw, n, w); }
    LONG    setLong(int n, LONG l)      { return SetWindowLong(_hw, n, l); }

    // Window positioning methods
    void    move(int x, int y, int cx, int cy, BOOL bRepaint)
                { MoveWindow(_hw, x, y, cx, cy, bRepaint); }
    void    setPos(HWND hwAfter, int x, int y, int cx, int cy, UINT wFlags)
                { SetWindowPos(_hw, hwAfter, x, y, cx, cy, wFlags); }
    void    setOrder(HWND hwAfter, UINT wFlags = 0)
                { SetWindowPos(_hw, hwAfter, 0, 0, 0, 0,
                    wFlags | SWP_NOMOVE | SWP_NOSIZE); }
    void    setSize(int cx, int cy, UINT wFlags = 0)
                { SetWindowPos(_hw, NULL, 0, 0, cx, cy,
                    wFlags | SWP_NOMOVE | SWP_NOZORDER); }

    // Other convenience methods
#if (WINVER < 0x030a)
    void    bringToTop(void)        { BringWindowToTop(_hw); }
#else
    BOOL    bringToTop(void)        { return BringWindowToTop(_hw); }
#endif
    void    close(void)             { CloseWindow(_hw); }
    BOOL    enable(BOOL bEnable)    { return EnableWindow(_hw, bEnable); }
    BOOL    flash(BOOL bInvert)     { return FlashWindow(_hw, bInvert); }
    HTASK   getTask(void)           { return GetWindowTask(_hw); }
    int     getText(LPSTR lpS, int cbMax)
                { return GetWindowText(_hw, lpS, cbMax); }
    int     getTextLength(void)     { return GetWindowTextLength(_hw); }
    void    invalidate(LPCRECT lpRect, BOOL bErase)
                { InvalidateRect(_hw, lpRect, bErase); }
    BOOL    isIconic(void)          { return IsIconic(_hw); }
    BOOL    is(void)                { return IsWindow(_hw); }
    BOOL    isEnabled(void)         { return IsWindowEnabled(_hw); }
    BOOL    isVisible(void)         { return IsWindowVisible(_hw); }
    BOOL    isZoomed(void)          { return IsZoomed(_hw); }
    BOOL    openIcon(void)          { return OpenIcon(_hw); }
    BOOL    postMessage(UINT wMsg, WPARAM wParam, LPARAM lParam)
                { return PostMessage(_hw, wMsg, wParam, lParam); }
    void    scroll(int x, int y, LPCRECT lpRect, LPCRECT lpClipRect)
                { ScrollWindow(_hw, x, y, lpRect, lpClipRect); }
    HWND    setActive(void)         { return SetActiveWindow(_hw); }
    void    setText(LPCSTR lpS)     { SetWindowText(_hw, lpS); }
    LRESULT sendMessage(UINT wMsg, WPARAM wParam, LPARAM lParam)
                { return SendMessage(_hw, wMsg, wParam, lParam); }
    BOOL    show(int nCmdShow)      { return ShowWindow(_hw, nCmdShow); }
    void    update(void)            { UpdateWindow(_hw); }
    void    validate(LPCRECT lpRect) { ValidateRect(_hw, lpRect); }

};

#endif // __WINDOW_HPP
