//
// HANDLER.CPP  Implementation of Handler class methods.
//
// Created:     Bruce Clift  7/21/91
//

#include <windows.h>
#pragma hdrstop

#include "handler.hpp"
#include "window.hpp"
#include "list.hpp"


/////////////////////////
// STATIC DATA MEMBERS //
/////////////////////////

FARPROC Handler::lpfnWPStd = NULL;      // Procedure instance of winProcStd
Handler *Handler::pNewHandler = NULL;   // Ptr to new handler for winProcReg


//
// The default constructor first allocates space for the callback method
// lists,   and if tables were passed as parameters it also builds them. It
// then initializes the default window procedure by setting it either to the
// standard Windows' DefWindowProc, DefFrameProc for MDI frame windows, or NULL
// if the handler is for a dialog box. 
//  INPUT: If pMsgList and/or pCmdList is non-NULL, they point to arrays with
// which to initialize the callback lists.  dpt is a flag which indicates
// whether the handler is for a window, dialog box, or MDI frame.
//
Handler::Handler(PCB pMsgList, PCB pCmdList, DEFPROCTYPE dpt)
{
    // Build message and command callback lists
    pMsgs = new List;
    if (pMsgList)
        regMsgCb(pMsgList);
    pCmds = new List;
    if (pCmdList)
        regCmdCb(pCmdList);

    // Set default window procedure (none for dialog boxes)
    switch (dpt) {
        default:
        case dptDialog:     defWinProc = (FARPROC) NULL;            break;
        case dptWindow:     defWinProc = (FARPROC) DefWindowProc;   break;
        case dptMDIFrame:   defWinProc = (FARPROC) DefFrameProc;    break;
        case dptMDIChild:   defWinProc = (FARPROC) DefMDIChildProc; break;
    }
}

//
// Destructor frees list memory.
//
Handler::~Handler()
{
    delete pMsgs;
    delete pCmds;
}


//
// regMsgCb and regCmdCb register callback methods in the message and command
// lists.  Since function member addresses may be more than 4 bytes, they must
// be allocated off the local heap.  The pointer to the allocated buffer is
// what is saved in the List objects' list.
//  INPUT: wMsg or wId is the Windows message or command ID to register.
// cb is the callback method address.  If bOverride is TRUE (default), then
// this callback will replace an existing callback.
//  RETURNS: TRUE if callback registered or overridden by a previous callback.
//
BOOL
Handler::regMsgCb(PCB pMsgList, BOOL bOverRide)
{
    for (PCB pcb=pMsgList; pcb->wIndex; pcb++)
        if (!regMsgCb(pcb->wIndex, pcb->cb, bOverRide))
            return FALSE;
    return TRUE;
}

BOOL
Handler::regCmdCb(PCB pCmdList, BOOL bOverRide)
{
    for (PCB pcb=pCmdList; pcb->wIndex; pcb++)
        if (!regCmdCb(pcb->wIndex, pcb->cb, bOverRide))
            return FALSE;
    return TRUE;
}

BOOL
Handler::regMsgCb(WORD wMsg, CBPROC cb, BOOL bOverRide)
{
    if (!pMsgs) return FALSE;

    // Free the old callback method address buffer and allocate a new one
    CBPROC *pCB = (CBPROC *) pMsgs->item(wMsg);
    if (pCB != NULL) {
        if (!bOverRide) return TRUE;
        LocalFree((HLOCAL) pCB);
    }
    pCB = (CBPROC *) LocalAlloc(LPTR, sizeof(CBPROC));

    // Store callback address in buffer and save pointer in list
    *pCB = cb;
    return pMsgs->insert(wMsg, (DWORD) (WORD) pCB, bOverRide);
}

BOOL
Handler::regCmdCb(WORD wId, CBPROC cb, BOOL bOverRide)
{
    if (!pCmds) return FALSE;

    // Free the old callback method address buffer and allocate a new one
    CBPROC *pCB = (CBPROC *) pCmds->item(wId);
    if (pCB != NULL) {
        if (!bOverRide) return TRUE;
        LocalFree((HLOCAL) pCB);
    }
    pCB = (CBPROC *) LocalAlloc(LPTR, sizeof(CBPROC));

    // Store callback address in buffer and save pointer in list
    *pCB = cb;
    return pCmds->insert(wId, (DWORD) (WORD) pCB, bOverRide);
}

CBPROC *
Handler::getMsgCB(WORD wMsg)
{
    return (CBPROC *) (*pMsgs ? pMsgs->item(wMsg) : NULL);
}

CBPROC *
Handler::getCmdCB(WORD wCmd)
{
    return (CBPROC *) (*pCmds ? pCmds->item(wCmd) : NULL);
}


///////////////////////////////
// SUPPORT FOR SUPERCLASSING //
///////////////////////////////

//
// Virtual methods to set/get other extra info (useful for superclassing).
//
void
Handler::setClsExtraBytes(HWND)
{}

void
Handler::getClsExtraBytes(HWND)
{}
