//
// MDICB.CPP    Callback methods for MDI classes.
//
// Created:     Bruce Clift  11/20/91
//
// Many of the MDI classes' features were inspired by
// Jeffrey Richter's Windows 3: A Developer's Guide.
//

#pragma hdrstop
#define OEMRESOURCE
#include <windows.h>

#include "mdi.hpp"
#include "handler.hpp"
#include "window.hpp"
#include "bitmap.hpp"
#include "rect.hpp"
#include "ribbon.hpp"
#include "statusba.hpp"
#include "menu.hpp"


////////////////////////////////////////////
// FRAME WINDOW MESSAGE CALLBACK METHODS //
///////////////////////////////////////////

//
// Registers the following methods as default message callbacks.
//
void
MDIFrame::regDefMsgs(Handler *pHand)
{
    static CB MsgCbList[] = {
        { WM_SIZE,              (CBPROC) &MDIFrame::wmSize },
        { WM_CLOSE,             (CBPROC) &MDIFrame::wmClose },
        { WM_QUERYENDSESSION,   (CBPROC) &MDIFrame::wmQueryEndSession },
        { WM_ENDSESSION,        (CBPROC) &MDIFrame::wmEndSession },
        { WM_SYSCOMMAND,        (CBPROC) &MDIFrame::wmSysCommand },
        { WM_NCLBUTTONDBLCLK,   (CBPROC) &MDIFrame::wmNCLButtonDblClk },
        { WM_COMMAND,           (CBPROC) &MDIFrame::wmCommand },
        { FM_CHILDDESTROY,      (CBPROC) &MDIFrame::fmChildDestroy },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);

    Window::regDefMsgs(pHand);      // Register base class' messages
}

//
// The WM_SIZE message calls the resizeClient method, which takes both
// a ribbon and status bar into account if they exist.
//
LRESULT
MDIFrame::wmSize(UINT, WPARAM, LPARAM)
{
    resizeClient();
    return 0L;
}

//
// Asks all MDI children if it is OK to shut down.  If all say OK,
// then the session is ended.
//
LRESULT
MDIFrame::wmClose(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    // Ask MDI children if OK to shut down
    // (TRUE means called by us and not Windows)
    if (sendMessage(WM_QUERYENDSESSION, TRUE, 0L)) {
        sendMessage(WM_ENDSESSION, TRUE, 0L);
        return callDefProc(wMsg, wParam, lParam);
    }
    return 0L;
}

//
// Asks each MDI child (ignoring iconic caption bars) whether it is OK
// to end the session.  If any child says no, the children who had
// already said OK are told that the session is NOT ending.
//
LRESULT
MDIFrame::wmQueryEndSession(UINT wMsg, WPARAM wParam, LPARAM)
{
    // Ask each child if it's OK to terminate
    Window aChild(hwClient(), GW_CHILD);
    for ( ; aChild; aChild.get(aChild.hwnd(), GW_HWNDNEXT)) {
        // Don't ask iconic caption bars
        if (GetWindow(aChild, GW_OWNER))
            continue;

        if (!aChild.sendMessage(wMsg, wParam, 0L))
            break;
    }

    // If everybody said OK, return TRUE to end session
    if (!aChild.hwnd())
        return TRUE;

    // If any child said no, tell other children NOT to terminate
    Window another(hwClient(), GW_CHILD);
    for ( ; another; another.get(another.hwnd(), GW_HWNDNEXT)) {
        // Stop when child that said "no" is reached
        if (aChild.hwnd() == another.hwnd())
            break;

        // Don't tell iconic caption bars
        if (GetWindow(another, GW_OWNER))
            continue;
        another.sendMessage(WM_ENDSESSION, FALSE, 0L);
    }
    return FALSE;
}

//
// Loop through all MDI children (ignoring iconic caption bars),
// telling each whether to end the session.
// INPUT: wParam is TRUE to end the session, FALSE to keep it going.
//
LRESULT
MDIFrame::wmEndSession(UINT wMsg, WPARAM wParam, LPARAM)
{
    Window aChild(hwClient(), GW_CHILD);
    for ( ; aChild; aChild.get(aChild.hwnd(), GW_HWNDNEXT)) {
        if (GetWindow(aChild, GW_OWNER))
            continue;
        aChild.sendMessage(wMsg, wParam, 0L);
    }
    return 0L;
}

//
// Sets the focus to the frame window, which in turn causes any
// comboboxes in the ribbon to be closed.
//
LRESULT
MDIFrame::wmSysCommand(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    SetFocus(hwnd());
    return callDefProc(wMsg, wParam, lParam);
}

//
// Allows double-clicking a maximized MDI child's system menu to close
// the child window.  (A Windows bug doesn't allow this).
//
LRESULT
MDIFrame::wmNCLButtonDblClk(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    if (wParam == HTMENU) {
        BOOL bMaximized;
        Window *pActive = getActiveChild(&bMaximized);

        if (bMaximized) {
            // Load the system close box bitmaps, then delete the bitmap object
            Bitmap bmClose(NULL, OBM_CLOSE);
            bmClose.destroy();
            bmClose.bmWidth /= 2;   // 2 close boxes in bitmap: use width / 2

            // Get dimensions of entire frame window, then narrow to close box
            Rect rc;
            GetWindowRect(hwnd(), &rc);
            rc.top += GetSystemMetrics(SM_CYCAPTION) +
                GetSystemMetrics(SM_CYFRAME);
            rc.bottom = rc.top + bmClose.bmHeight;
            rc.left += GetSystemMetrics(SM_CXFRAME);
            rc.right = rc.left + bmClose.bmWidth;

            // If cursor is within close box, tell MDI child to close
            if (rc.ptIn(MAKEPOINT(lParam))) {
                pActive->sendMessage(WM_SYSCOMMAND, SC_CLOSE, lParam);
                return 0L;
            }
        }
    }
    return callDefProc(wMsg, wParam, lParam);
}

//
// This method is called only if no match is found in the command
// callback list for the command in wParam.  It passes menu items in
// the "Window" menu on to Windows.  Other items are funnelled to the
// active child.
//
LRESULT
MDIFrame::wmCommand(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    // Let Windows handle child activation in the "Window" menu
    if (wParam >= getFirstChild())
        return callDefProc(wMsg, wParam, lParam);

    // Pass command on to active child
    Window *pActive = getActiveChild();
    if (pActive)
        return (pActive->sendMessage(wMsg, wParam, lParam));

    return 0L;
}

//
// Destroy a child window object after the window has been destroyed.
//
LRESULT
MDIFrame::fmChildDestroy(UINT, WPARAM, LPARAM lParam)
{
    MDIChild *lpCh = (MDIChild *) lParam;
    lpCh->destroy();
    return 0L;
}


//////////////////////////////////
// FRAME WINDOW UTILITY METHODS //
//////////////////////////////////

//
// Get a pointer to the active child window.  If pbMax is non-NULL, it
// is filled with the maximized flag from the high-order word.
//
Window *
MDIFrame::getActiveChild(BOOL *pbMax)
{
    DWORD dwActive = SendMessage(hwClient(), WM_MDIGETACTIVE, 0, 0L);
    if (pbMax)
        *pbMax = HIWORD(dwActive);
    return (findWinHnd(LOWORD(dwActive)));
}

//
// Resizes the client window.  Called when the frame window is
// resized, or the status bar or ribbon is toggled.
//
void
MDIFrame::resizeClient(void)
{
    // Get dimensions of frame client area
    Rect rcFr(hwnd());

    // Make adjustment for ribbon
    Ribbon *pRibbon = getRibbon();
    if (pRibbon && pRibbon->is() && pRibbon->isVisible()) {
        Rect rcRbn(pRibbon->hwnd());
        rcFr.top    += rcRbn.bottom;
        rcFr.bottom -= rcRbn.bottom;
    }

    // Make adjustment for status bar
    StatusBar *pSB = getStatusBar();
    if (pSB && *pSB) {
        Rect rcSB;
        pSB->getRect(rcSB);
        rcFr.bottom -= rcSB.bottom - rcSB.top;
    }
    
    // Size the client
    MoveWindow(hwClient(),
        rcFr.left, rcFr.top, rcFr.right, rcFr.bottom, TRUE);
}

//
// Allows frame and child windows to change the frame window's
// main menu.
//
void
MDIFrame::changeMDIMenu(HMENU hm, WORD wID)
{
    // Locate popup containing item ID in "Window" popup
    HMENU hSub;
    for (WORD wCount = GetMenuItemCount(hm); wCount; wCount--) {
        hSub = GetSubMenu(hm, wCount - 1);
        if ((int) GetMenuState(hSub, wID, MF_BYCOMMAND) != -1)
            break;
    }
    
    // Tell client window to set up new menu, then draw it
    SendMessage(_hwClient, WM_MDISETMENU, 0, MAKELONG(hm, hSub));
    DrawMenuBar(hwnd());
}


///////////////////////////////////////////
// CHILD WINDOW MESSAGE CALLBACK METHODS //
///////////////////////////////////////////

//
// Registers the following methods as default message callbacks.
//
void
MDIChild::regDefMsgs(Handler *pHand)
{
    static CB MsgCbList[] = {
        { WM_CLOSE,     (CBPROC) &MDIChild::wmClose },
        { WM_DESTROY,   (CBPROC) &MDIChild::wmDestroy },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);

    Window::regDefMsgs(pHand);      // Register base class' messages
}

//
// Send a message to self to see if we can close the window, then send
// another to indicate that we are closing.
//
LRESULT
MDIChild::wmClose(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    if (sendMessage(WM_QUERYENDSESSION, TRUE, 0L)) {
        sendMessage(WM_ENDSESSION, TRUE, 0L);
        callDefProc(wMsg, wParam, lParam);
    }
    return 0L;
}

//
// Sends message to frame window that a child has been destroyed.
//
LRESULT
MDIChild::wmDestroy(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    if (pFrame == NULL)
        return 0L;

    pFrame->postMessage(FM_CHILDDESTROY, hwnd(),
        (LPARAM) (MDIChild *) this);
    callDefProc(wMsg, wParam, lParam);
    return 0L;
}


//////////////////////////////////////////////
// CHILD WINDOW STATUS BAR CALLBACK METHODS //
//////////////////////////////////////////////

//
// Registers some callback methods useful for the status bar.
//
void
MDIChild::regSBMsgs(Handler *pHand)
{
    static CB MsgCbList[] = {
        { WM_MENUSELECT, (CBPROC) &MDIChild::wmMenuSelect },
        { WM_ENTERIDLE,  (CBPROC) &MDIChild::wmEnterIdle },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);
}

//
// User is scrolling through the child window's menu.
//
LRESULT
MDIChild::wmMenuSelect(UINT, WPARAM wParam, LPARAM lParam)
{
    // Get pointer to frame window's status bar
    StatusBar *pSB = (pFrame ? pFrame->getStatusBar() : getStatusBar());
    if (!pSB || !*pSB) return 0L;

    // Has user stopped using menus?
    if (lParam == MAKELONG(-1, 0)) {
        pSB->setMenuHelpIndex(NULL, 0);
        return 0L;
    }

    // Get type of menu
    int iHelp = 0;
    switch (LOWORD(lParam) & (MF_POPUP | MF_SYSMENU)) {
    case 0: {                        // Menu item NOT on app's system menu
        // Provide same help for all items in frame's "Window" menu
        WORD wFirstChild = pFrame->getFirstChild();
        if ((wParam > wFirstChild) && (wParam <= wFirstChild + 9))
            wParam = wFirstChild;
        iHelp = pSB->getMenuHelpIndex(this, wParam);
        break; }

    case MF_POPUP: {                // wParam is handle to popup menu
        // Calculate index of top-level menu
        Menu m(pFrame->hwnd());
        WORD w = m.getItemCount();
        while (w--)
            if (GetSubMenu(m, w) == (HMENU) wParam)
                break;
        if (isZoomed()) w++;            // Jump over maximized system menu
        iHelp = pSB->getPopupHelpIndex(this, w);
        break; }

    case MF_SYSMENU:                 // Frame's system menu item
        iHelp = pSB->getSysMenuHelpIndex(this, (wParam & 0x0fff) >> 4);
        break; 

    case MF_POPUP | MF_SYSMENU:     // wParam is handle to app's system menu
        iHelp = pSB->getPopupHelpIndex(this, 0);
        break; 
    }

    // Save the menu help index for later use
    if (iHelp)
        pSB->setMenuHelpIndex(this, iHelp);
    return 0L;
}

//
// Passes the WM_ENTERIDLE message on to the frame window.
//
LRESULT
MDIChild::wmEnterIdle(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    if (pFrame)
        pFrame->sendMessage(wMsg, wParam, lParam);
    return 0L;
}
