//
// MDIVERT.CPP  Tile MDI children vertically.
//
// Created:     Bruce Clift  11/20/91
//
// This MDI feature was inspired by Jeffrey Richter's
// Windows 3: A Developer's Guide.
//

#include <windows.h>
#pragma hdrstop

#include "mdi.hpp"
#include "rect.hpp"
#include "window.hpp"
#include "point.hpp"

#define min(x,y) ((x) < (y) ? (x) : (y))
#define max(x,y) ((x) > (y) ? (x) : (y))


//
// Tile all open MDI children vertically (instead of standard Windows
// horizontal tiling).
//
void
MDIFrame::tileVertically(void)
{
    // Force scroll bars off to get full client size
    ShowScrollBar(_hwClient, SB_BOTH, FALSE);

    // Arrange icons at bottom of client area
    SendMessage(_hwClient, WM_MDIICONARRANGE, 0, 0L);

    Rect rc;
    int nOpenMDIChildren = 0;
    int nTopOfBottomIconRow = 0;
    Window aChild(_hwClient, GW_CHILD);
    do {
        // Ignore icon captions
        if (GetWindow(aChild, GW_OWNER))
            continue;
        if (!aChild.isIconic())         // Count open child windows
            nOpenMDIChildren++;
        else {                          // Keep track of top of icons
            rc.getWindow(aChild);
            if (rc.top > nTopOfBottomIconRow)
                nTopOfBottomIconRow = rc.top;
        }
    } while (aChild.get(aChild.hwnd(), GW_HWNDNEXT));
    if (nOpenMDIChildren == 0)          // No tiling needed if none open
        return;

    // Get height of usable client area for tiling
    rc.getClient(_hwClient);
    if (nTopOfBottomIconRow) {          // At least one child is iconic
        Point p(0, nTopOfBottomIconRow);
        p.screenToClient(_hwClient);
        rc.bottom = p.y;
    }

    // Restore the active MDI child if it's maximized
    BOOL bMaximized;
    Window *pActive = getActiveChild(&bMaximized);
    if (bMaximized)
        pActive->show(SW_RESTORE);

    // Calculate the height needed for each row
    int nMinWndHeight = max(1, rc.bottom /
        (5 * GetSystemMetrics(SM_CYCAPTION)));
    int nNumRows = min(nOpenMDIChildren, nMinWndHeight);
    int nRowHeight = rc.bottom / nNumRows;

    // Loop through open child windows, repositioning each
    aChild.get(_hwClient, GW_CHILD);
    HDWP hWinPosInfo = BeginDeferWindowPos(nOpenMDIChildren);

    int nNumWndsOnRow, nColWidth;
    for (int nRow=0; nRow < nNumRows; nRow++) {
        // Calculate number of MDI children on this row and their column width
        nNumWndsOnRow = nOpenMDIChildren / nNumRows;
        if (nOpenMDIChildren % nNumRows > (nNumRows - (nRow + 1)))
            nNumWndsOnRow++;
        nColWidth = rc.right / nNumWndsOnRow;

        // Loop through children, ignoring icons & caption bars
        for (int nCol=0; nCol < nNumWndsOnRow; ) {
            if (!aChild.isIconic() && !GetWindow(aChild, GW_OWNER)) {
                // Reposition child window
                hWinPosInfo = DeferWindowPos(hWinPosInfo, aChild, NULL,
                    nCol * nColWidth, nRow * nRowHeight,
                    nColWidth, nRowHeight,
                    SWP_NOACTIVATE | SWP_NOZORDER);
                nCol++;
            }
            aChild.get(aChild, GW_HWNDNEXT);
        }
    }

    // Update all windows at once
    EndDeferWindowPos(hWinPosInfo);
}

