//
// MODAL.CPP    Implementation of Modal dialog box class.
//
// Created:     Bruce Clift  10/20/91
//

#include <windows.h>
#pragma hdrstop

#include "modal.hpp"
#include "handler.hpp"
#include "dialog.hpp"


//
// Default constructor just calls Dialog constructor.
//
Modal::Modal() : Dialog()
{}


//
// run executes a modal dialog box.  A pointer the Handler object must be
// saved in the base Window class for the winProcStd routine to find.  Also,
// a pointer to this Modal object is saved in Window's static member pNewWin
// so that the new dialog box handle may be added to the window handle list.
// 
//  INPUT: hInst is the application's instance handle.  lpszTmplt is the
// resource string of the dialog box template (for DialogBox).  hwPar is the
// parent window handle.  dwParam is an optional parameter which would be
// passed in the WM_INITDIALOG message.
//
int
Modal::run(HINSTANCE hInst, LPCSTR lpszTmplt, HWND hwPar, LPARAM lParam)
{
    // Save Handler ptr in Window for winProcStd to find
    Handler *pHand = getHandler();
    setHandler(pHand);
    regDefMsgs(pHand);

    createChildren();   // Create any child control objects
    pNewWin = this;     // Save this ptr in Window static data member

    // Run the dialog box (getting a procedure instance for winProcStd)
    f.created = FALSE;
    int nRet = DialogBoxParam(hInst, lpszTmplt, hwPar,
        pHand->getWinProcStd(hInst), lParam);

    destroy();          // Remove dialog from window handle list
    return nRet;
}


/////////////////////////////////////////////
// DEFAULT MESSAGES FOR MODAL DIALOG BOXES //
/////////////////////////////////////////////

//
// regDefMsgs registers standard Modal dialog box callback methods.
//
void
Modal::regDefMsgs(Handler *pHand)
{
    static CB MsgCbList[] = {
        { WM_INITDIALOG, (CBPROC) &Modal::wmInitDialog },
        { 0, NULL }
    };
    static CB CmdCbList[] = {
        { IDOK,     (CBPROC) &Modal::idOk },
        { IDCANCEL, (CBPROC) &Modal::idCancel },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);
    pHand->regCmdCb(CmdCbList, FALSE);

    // Register base class' default messages
    Dialog::regDefMsgs(pHand);
}

//
// wmInitDialog is called when a dialog box is about to be displayed.
// Returning TRUE indicates that the focus has not been set to any
// of the controls in the dialog.
//
LRESULT
Modal::wmInitDialog(UINT, WPARAM, LPARAM)
{
    return TRUE;
}

//
// idOk is called when the OK button in a Modal dialog box is pushed.
//
LRESULT
Modal::idOk(UINT, WPARAM, LPARAM)
{
    EndDialog(hwnd(), TRUE);
    return 0L;
}

//
// idCancel is called when the Cancel button is pushed, or the Close option
// in the system menu is selected.
//
LRESULT
Modal::idCancel(UINT, WPARAM, LPARAM)
{
    EndDialog(hwnd(), FALSE);
    return 0L;
}
