//
// MODELESS.CPP Implementation of Modeless dialog boxes.
//
// Created:     Bruce Clift  10/20/91
//

#include <windows.h>
#pragma hdrstop

#include "modeless.hpp"
#include "handler.hpp"
#include "dialog.hpp"


//
// Default constructor just calls Dialog constructor.
//
Modeless::Modeless() : Dialog()
{}

//
// create creates a modeless dialog box.  A pointer the Handler object must be
// saved in the base Window class for the winProcStd routine to find.  Also, a
// pointer to this Modeless object is saved in Window's static member pNewWin
// so that the new dialog box handle may be added to the window handle list.
//
//  INPUT: hInst is the application's instance handle.  lpszTmplt is the
// resource string of the dialog box template (for CreateDialog).  hwPar is the
// parent window handle.  lParam is an optional parameter which would be
// passed in the WM_INITDIALOG message.
//
HWND
Modeless::create(HINSTANCE hInst, LPCSTR lpszTmplt, HWND hwPar, LPARAM lParam)
{
    // Save Handler ptr in Window for winProcStd to find
    Handler *pHand = getHandler();
    setHandler(pHand);
    regDefMsgs(pHand);

    createChildren();   // Create any child control objects first
    pNewWin = this;     // Save this ptr in Window static data member

    // Create the dialog box (getting a procedure instance for winProcStd)
    if (CreateDialogParam(hInst, lpszTmplt, hwPar,
            pHand->getWinProcStd(hInst), lParam)) {
        f.created = TRUE;   // Flag that it's a modeless box

        // Display dialog only if requested (default TRUE)
        if (createVisible()) {
            show(SW_SHOWNORMAL);
            update();
        }
    }
    else {                  // If not successful in creating dialog
        destroyChildren();  // destroy child control objects
    }
    return (hwnd());
}

//
// By default, create visible modeless dialog boxes.
//
BOOL
Modeless::createVisible(void)
{
    return TRUE;
}


//
// regDefMsgs registers standard Modeless dialog box callback methods.
//
void
Modeless::regDefMsgs(Handler *pHand)
{
    static CB CmdCbList[] = {
        { IDOK,     (CBPROC) &Modeless::idOk },
        { IDCANCEL, (CBPROC) &Modeless::idCancel },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regCmdCb(CmdCbList, FALSE);

    // Register base class' default messages
    Dialog::regDefMsgs(pHand);
}

//
// idOk is called when the OK button in a Modeless dialog box is pushed.
//
LRESULT
Modeless::idOk(UINT, WPARAM, LPARAM)
{
    destroy();
    return TRUE;
}

//
// idCancel is called when the Cancel button is pushed, or the Close option
// in the system menu is selected.
//
LRESULT
Modeless::idCancel(UINT, WPARAM, LPARAM)
{
    destroy();
    return FALSE;
}
