//
// PROFILE.CPP  Windows .ini file support.
//
// Created:     Bruce Clift     11/22/91
//              Greg Voss       5/10/93   adapt for MSC 7.0
//

#include <windows.h>
#pragma hdrstop
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>

#include "profile.hpp"
#include "file.hpp"

#define min(x,y) ((x) < (y) ? (x) : (y))
#define max(x,y) ((x) > (y) ? (x) : (y))


/////////////////////////
// STATIC DATA MEMBERS //
/////////////////////////

char *Profile::szWinIni = "win.ini";
char *Profile::szDefDelim = ",";


/////////////////////////
// FILE INITIALIZATION //
/////////////////////////

//
// Default constructor clears instance variables
//
Profile::Profile() : hMem(NULL), wLinePos(0), wFileLen(0)
{}

//
// The init method opens a .ini file in preparation for getting or setting
// values.  If bPreLoad is TRUE, the entire contents is read immediately
// (recommended if the file is on a floppy).
//
BOOL
Profile::init(char *szPath, BOOL bPreLoad)
{
    // Free any previous file data buffer
    if (hMem) {
        GlobalFree(hMem);
        hMem = NULL;
    }

    // Use default file or .ini extension as needed
    BOOL bExt = FALSE;
    char *pExt = szLine +
        wsprintf(szLine, (LPSTR) (szPath && *szPath ? szPath : szWinIni));
    while (pExt-- > szLine) {
        switch (*pExt) {
            case '.':   bExt = TRUE; break;
            case '\\':
            case ':':   break;
            default:    continue;
        }
        break;
    }
    if (!bExt) lstrcat(szLine, ".ini");

    // Open the file and preload if requested
    f.ok = open(szLine);
    if (f.ok && bPreLoad) {
        hMem = GlobalAlloc(GHND, wFileLen);
        LPSTR lpFile = GlobalLock(hMem);
        if (read(lpFile, wFileLen) != wFileLen)
            f.ok = FALSE;
        GlobalUnlock(hMem);
        if (!f.ok) {                // Free memory on failure
            GlobalFree(hMem);
            hMem = NULL;
        }
    }
    close();
    f.preLoad = (bPreLoad ? 1 : 0);
    return (f.ok);
}


/////////////////////////////////////////
// METHODS TO GET AND SET PROFILE DATA //
/////////////////////////////////////////

//
// These two methods are used to write an integer or string value into a
// profile file.  A new [szApp] section and a new szKey= line is started
// as needed.  If szApp is NULL, the line is added at the current location.
//
BOOL
Profile::setInt(char *szApp, char *szKey, int nVal)
{
    // Operation not allowed for preloaded profiles
    if (f.preLoad) return FALSE;

    char *buf = new char [10];
    wsprintf(buf, "%d", nVal);
    BOOL bOK = setString(szApp, szKey, buf);
    delete buf;
    return bOK;
}

BOOL
Profile::setString(char *szApp, char *szKey, char *szVal)
{
    // Operation not allowed for preloaded profiles
    if (f.preLoad) return FALSE;

    if (!f.ok || !reopen())             // Reopen file for search
        { return (f.ok = FALSE); }
    BOOL bFound = (szApp == NULL);      // Don't search if section not given
    if (!bFound && !findApp(szApp)) {   // If section not found
        newApp(szApp);                  // start the section
    } else if (!bFound && findKey(szKey)) { // Find key if section was found
        bFound = TRUE;
    } else {                            // Entering a new line in file
        do {
            seek(--wLinePos);           // Skip back over blank lines
            read(szLine, 1);
        } while (szLine[0] <= ' ');
        seek(wLinePos += 3);
    }
    newLine(!bFound, "%s=%s\r\n", szKey, szVal);
    close();
    return TRUE;
}


//
// Returns an integer value from a profile file.
//
int
Profile::getInt(char *szApp, char *szKey, int nDef)
{
    // Find the requested section and key (return default if not found)
    if (!f.ok || !reopen())
        { f.ok=FALSE; return (0); }
    BOOL bFound = (findApp(szApp) && findKey(szKey));
    close();

    return (bFound ? atoi(getValPtr()) : nDef);
}

//
// Get a string value from a profile file.  Note that the string for the
// obtained value may be NULL.  This should be the case when the getField
// method is to be used.
// RETURNS: number of characters written out.
//
int
Profile::getString(char *szApp, char *szKey, char *szDef, char *szVal, int len)
{
    // Find the requested section and key (return default if not found)
    if (!f.ok || !reopen())
        { f.ok=FALSE; return (0); }
    BOOL bFound = (findApp(szApp) && findKey(szKey));
    close();

    // Result not wanted, not found?
    if (!szVal || !len) return (0);
    else if (!bFound)   return (wsprintf(szVal, szDef ? szDef : ""));

    // Limit the size of the string returned
    char *pVal = getValPtr();
    if (--len < lstrlen(pVal))
        pVal[len] = 0;
    return (wsprintf(szVal, pVal));
}

//
// getField should be called after a successful call to getString
// to break up a string into fields separated by the given delimeter.
// RETURNS: number of characters written out.
//
int
Profile::getField(char *szVal, int len, char *szDelim)
{
    if (!szVal) return 0;

    // Use the strtok routine to get fields from the current line
    char *p = strtok((f.fresh ? getValPtr() : NULL), szDelim);
    f.fresh = 0;
    if (!p) {
        *szVal = 0;
        return 0;
    }

    // Don't overflow the caller's string
    if (lstrlen(p) > --len)
        p[len] = 0;
    return (wsprintf(szVal, p));
}


//////////////////////////////////////
// METHODS TO FIND DATA IN THE FILE //
//////////////////////////////////////

//
// Finds the section in the file labelled [szApp].
//
BOOL
Profile::findApp(char *szApp)
{
    // Build section string
    if (!szApp || !*szApp) return FALSE;
    char *buf = new char [lstrlen(szApp) + 3];
    wsprintf(buf, "[%s]", (LPSTR) szApp);

    // Find section, starting from beginning if required
    long currPos = position();
    BOOL bFound = findKey(buf, TRUE);
    if (!bFound && currPos) {       // Don't bother if we were at beginning
        seek(0L);
        bFound = findKey(buf, TRUE);
    }
    delete buf;
    return bFound;
}

//
// Finds the next profile data with the given key.
//
BOOL
Profile::findKey(char *szKey, BOOL bApp)
{
    while (getLine()) {                 // Get lines until match found
        //if (*szKey && !strncmpi(szLine, szKey, lstrlen(szKey)))
#if (0 MSC(+1))         // Microsoft C++
        if (*szKey && !strnicmp(szLine, szKey, lstrlen(szKey)))
#elif (0 BCC(+1))       // Borland C++
        if (*szKey && !strncmpi(szLine, szKey, lstrlen(szKey)))
#endif
            return TRUE;
        if (!bApp && *szLine == '[')        // Stop when next section found
            return FALSE;
    }
    return FALSE;
}

//
// Returns a pointer to the position in the current line buffer immediately
// following the equal sign.  This is where the value is to be found.
// THIS METHOD WILL NEVER RETURN NULL.
//
char *
Profile::getValPtr(void)
{
    char *p = strchr(szLine, '=');
    return (p ? p+1 : strchr(szLine, 0));
}


////////////////////////////////////////
// METHODS TO INSERT DATA IN THE FILE //
////////////////////////////////////////

//
// Starts a new section [szApp] in the file.
//
void
Profile::newApp(char *szApp)
{
    seek(0L, 2);
    newLine(TRUE, "\r\n[%s]\r\n", szApp);
}

//
// Insert a formatted string into the file.  If bNew is FALSE, the current line
// is replaced in the file with the given line.
//
void
Profile::newLine(BOOL bNew, char *szFmt, ...)
{
    // Format the new line
    va_list ap;
    va_start(ap, szFmt);
    int len = vsprintf(szLine, szFmt, ap);
    va_end(ap);

    // Write a new line after the current line
    if (bNew)
        wLinePos = position();

    // Read remainder of file into a temporary buffer
    WORD wSize = wFileLen - wLinePos;
    hMem = GlobalAlloc(GHND, wSize);
    LPSTR lpFile = GlobalLock(hMem);
    wSize = read(lpFile, wSize);

    // Write the new line into the file
    seek(wLinePos);
    write(szLine, len);
    
    // Write the remainder of the file after the new line
    write(lpFile, wSize);
    GlobalUnlock(hMem);
    GlobalFree(hMem);

    // Position the file pointer after the new line
    seek(wLinePos + len);
    long lLen = length();
    wFileLen = ((lLen >= MAXFILELEN) ? -1 : (WORD) lLen);
}


/////////////////////
// FILE OPERATIONS //
/////////////////////

//
// Open the file for the first time.
//
BOOL
Profile::open(LPCSTR szPath, UINT wFlags)
{
    // Open the file
    if (!File::open(szPath, wFlags))
        return FALSE;

    // Store the file's size when first opening
    long lLen = length();
    wFileLen = ((lLen >= MAXFILELEN) ? -1 : (WORD) lLen);
    if ((int) wFileLen < 0)
        close();
    wFilePos = 0;

    return (hFile >= 0);
}

//
// read, write, and seek operations also work for preloaded files.
// The current file position is kept in wFilePos.
//
UINT
Profile::read(HPVOID lpBuffer, UINT wBytes)
{
    if (f.preLoad) {
        wBytes = min(wBytes, wFileLen - wFilePos);
        LPSTR lpFile = GlobalLock(hMem);
        _fmemcpy(lpBuffer, lpFile + wFilePos, wBytes);
        GlobalUnlock(hMem);
    } else {
        wBytes = File::read(lpBuffer, wBytes);
    }
    wFilePos += wBytes;
    return wBytes;
}

UINT
Profile::write(HPCVOID lpBuffer, UINT wBytes)
{
    // Not allowed to write to a pre-loaded file
    if (f.preLoad)
        return 0;

    wBytes = File::write(lpBuffer, wBytes);
    wFilePos += wBytes;
    return wBytes;
}

long
Profile::seek(long lOffset, int iOrigin)
{
    // Do normal seek for non-preloaded files
    if (!f.preLoad)
        return (wFilePos = File::seek(lOffset, iOrigin));

    // Change file position for preloaded files
    switch (iOrigin) {
        case 0: wFilePos  = lOffset; break;
        case 1: wFilePos += lOffset; break;
        case 2: wFilePos  = lOffset + wFileLen; break;
    }
    wFilePos = max(0, (int) min(wFilePos, wFileLen));
    return ((long) wFilePos);
}


//
// Read a single line from the current location.  The line is read into an
// instance variable.  The file pointer is positioned after the line, and
// the position where the line started is saved in an instance variable.
// Returns FALSE on end-of-file.
//
BOOL
Profile::getLine(void)
{
    // Save position at the start of this line
    wLinePos = position();

    // Read a line from the file
    int nRead = read(szLine, MAXLINELEN);
    szLine [nRead] = 0;
    if (!nRead) return FALSE;

    // Find CR and/or LF, set file pointer, and terminate the line
    for (int i=0; i<nRead; i++) {
        if (szLine[i] != '\r' && szLine[i] != '\n')
            continue;
        seek(wLinePos + i +
            ((szLine[i]=='\r' && szLine[i+1]=='\n') ? 2 : 1));
        szLine[i] = 0;
        break;
    }

    f.fresh = 1;        // Flag that we have a fresh line
    return (TRUE);
}
