//
// RIBBON.CPP   Implementation for Ribbon modeless dialog box class.
//
// Created:     Bruce Clift  11/9/91
//

#include <windows.h>
#pragma hdrstop

#include "ribbon.hpp"
#include "handler.hpp"
#include "paint.hpp"
#include "pen.hpp"
#include "rect.hpp"


/////////////////////////
// DIALOG BOX CREATION //
/////////////////////////

//
// Default constructor simply calls Modeless constructor.
//
Ribbon::Ribbon() : Modeless()
{}


//
// The creation method gets the template name from the virtual method,
// and calls the modeless creation method to create the dialog box.
//
HWND
Ribbon::create(HWND hwParent, HINSTANCE hInst)
{
    if (!hInst)
        hInst = GetWindowWord(hwParent, GWW_HINSTANCE);
    return Modeless::create(hInst, getTemplate(), hwParent);
}

//
// Don't automatically make ribbons visible.
//
BOOL
Ribbon::createVisible(void)
{
    return FALSE;
}

//
// Allows derived classes to set template name.
//
char *
Ribbon::getTemplate(void)
{
    return ("Ribbon");
}


/////////////////////////////
// RIBBON CALLBACK METHODS //
/////////////////////////////

//
// regDefMsgs registers special ribbon callback methods.
//
void
Ribbon::regDefMsgs(Handler *pHand)
{
    static CB MsgCbList[] = {
        { WM_ENABLE, (CBPROC) &Ribbon::wmEnable },
        { WM_PAINT,  (CBPROC) &Ribbon::wmPaint },
        { 0, NULL }
    };
    static CB CmdCbList[] = {
        { IDOK,     (CBPROC) &Ribbon::idOkCancel },
        { IDCANCEL, (CBPROC) &Ribbon::idOkCancel },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);
    pHand->regCmdCb(CmdCbList, FALSE);

    // Register base class' default messages
    Modeless::regDefMsgs(pHand);
}

//
// Sets all child windows to the same status as the ribbon.
//
LRESULT
Ribbon::wmEnable(UINT, WPARAM wParam, LPARAM)
{
    for (Window ch(hwnd(), GW_CHILD); ch; ch.get(ch.hwnd(), GW_HWNDNEXT))
        ch.enable(wParam);
    return 0L;
}

//
// Paints a dividing line between ribbon and rest of parent's client area.
//
LRESULT
Ribbon::wmPaint(UINT, WPARAM, LPARAM)
{
    // Create device context and pen with the width of a window's border
    Paint p(hwnd());
    int cyBorder = GetSystemMetrics(SM_CYBORDER);
    Pen aPen(PS_SOLID, cyBorder, RGB(0,0,0));
    aPen.select(p);

    // Draw line below ribbon
    Rect rc(hwnd());
    MoveTo(p, 0, rc.bottom - cyBorder);
    LineTo(p, rc.right, rc.bottom - cyBorder);

    // Destroy the device context and the pen
    p.destroy();
    aPen.destroy();
    return 0L;
}

//
// Rather than destroying the ribbon, focus is returned to the parent
// window if OK or Cancel is received.
//
LRESULT
Ribbon::idOkCancel(UINT, WPARAM, LPARAM)
{
    SetFocus(GetParent(hwnd()));
    return 0L;
}

