//
// STATUSBA.CPP Implementation of StatusBar class.
//
// Created:     Bruce Clift  11/8/91
//

#include <windows.h>
#pragma hdrstop

#include "statusba.hpp"
#include "window.hpp"
#include "handler.hpp"
#include "paint.hpp"
#include "devconte.hpp"
#include "textmetr.hpp"
#include "pen.hpp"
#include "sbhelp.h"


//
// Default constructor sets window handle owner.
//
StatusBar::StatusBar()
    : pOwner(NULL), pHelper(NULL), _iHelp(0)
{}


/////////////////////////////
// CALLBACK HELPER METHODS //
/////////////////////////////

//
// The set method is called to toggle the status bar on or off.
// It saves the Window object pointer in an instance variable and
// registers messages with the Window's Handler object.
//
void
StatusBar::set(Window *pWin)
{
    static CB MsgCbList[] = {
        { WM_PAINT,         (CBPROC) &Window::wmPaint },
        { WM_MENUSELECT,    (CBPROC) &Window::wmMenuSelect },
        { WM_ENTERIDLE,     (CBPROC) &Window::wmEnterIdle },
        { 0, NULL }
    };

    // If turning the status bar off, do nothing else
    if (!(pOwner = pWin)) return;

    // Register messages with Handler object
    Handler *pHand = pOwner->getHandler();
    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);

    // Invalidate status bar region
    invalidate();
}

//
// This method is called by the owner Window's wmPaint method.
// If the Window is already processing the WM_PAINT message
// (as is often the case), this method should be called by that
// wmPaint method instead.
//
void
StatusBar::wmPaint(Paint &p)
{
    // Get help from active MDI child (or from owner Window if none)
    if (!pOwner) return;
    pHelper = pOwner->getActiveChild();
    if (!pHelper) pHelper = pOwner;

    // Get help string and paint it
    char *pHelp = getHelp(getHelpIndex(pHelper));
    if (pHelp)
        textOut(p, pHelp);
    pHelper = NULL;
}

//
// This method is called by the Window's wmEnterIdle method.
//
void
StatusBar::wmEnterIdle(void)
{
    if (!pHelper) return;           // Help already displayed

    // Get menu help string and pointer to status bar
    char *pHelp = getHelp();
    pHelper = NULL; // Flag that we've given help for current menu item

    // Invalidate status bar and write out the text
    if (pHelp) {
        invalidate();
        Paint p(*pOwner);
        textOut(p, pHelp);
    }
}


//////////////////////////////////////////////////
// GETTING AND SETTING STATUS BAR HELP MESSAGES //
//////////////////////////////////////////////////

//
// Sets the window providing menu help and the help index.
//
void
StatusBar::setMenuHelpIndex(Window *pWin, int i)
{
    // Set the instance variables
    pHelper = pWin;
    _iHelp = i;

    if (pWin == NULL)   // When user finishes scrolling through menu
        invalidate();   // force status bar update
}

//
// These methods may be overriden either in a derived StatusBar OR
// Window object to supply help string indexes. The Window parameter
// is given the first opportunity, and finally the status bar itself.
// getHelpIndex is for non-menu help.  getSys/Menu/PopupHelpIndex
// is for menu item and popup help strings.
//

int
StatusBar::getHelpIndex(Window *pWin)
{
    int iRet;
    if (pWin && (iRet=pWin->getHelpIndex()))
        return iRet;
    return 0;
}

int
StatusBar::getMenuHelpIndex(Window *pWin, int i)
{
    int iRet;
    if (pWin && (iRet=pWin->getMenuHelpIndex(i)))
        return iRet;
    return 0;
}

int
StatusBar::getSysMenuHelpIndex(Window *pWin, int i)
{
    int iRet;
    if (pWin && (iRet=pWin->getSysMenuHelpIndex(i)))
        return iRet;
    return (IDS_SYSMENUID + i);
}

int
StatusBar::getPopupHelpIndex(Window *pWin, int i)
{
    int iRet;
    if (pWin && (iRet=pWin->getPopupHelpIndex(i)))
        return iRet;
    return 0;
}

//
// Convert the saved help index into a help string.  As with the
// methods above, the helper window, then the owner window are
// given the first opportunity to supply the help string.
//
char *
StatusBar::getHelp(int i)
{
    char *pHelp;
    if ((pHelper && (pHelp=pHelper->getHelp(i))) ||
        (pOwner && (pHelp=pOwner->getHelp(i))))
            return pHelp;

    // Load help string resource
    static char szBuf[80];
    LoadString(pOwner->getWord(GWW_HINSTANCE), i, szBuf, 80);
    return szBuf;
}

char *
StatusBar::getHelp(void)
{
    char *pHelp;
    if ((pHelper && (pHelp=pHelper->getHelp())) ||
        (pOwner && (pHelp=pOwner->getHelp())))
            return pHelp;

    return getHelp(_iHelp);
}


///////////////////////////////
// DISPLAYING THE STATUS BAR //
///////////////////////////////

//
// The getRect method returns the size of the status bar.
//
void
StatusBar::getRect(RECT &r)
{
    if (!pOwner) {              // User has turned status bar off
        SetRectEmpty (&r);      // no status bar
        return;
    }
    GetClientRect(*pOwner, &r);

    DevContext dc(*pOwner);
    TextMetric tm(dc);
    r.top = r.bottom - tm.tmHeight - GetSystemMetrics(SM_CYBORDER);
}

//
// Invalidates the status bar rectangle on the screen.
//
void
StatusBar::invalidate(BOOL bErase)
{
    if (!pOwner) return;
    RECT r;
    getRect(r);
    pOwner->invalidate(&r, bErase);
}

//
// Display a string in the status bar.
//
void
StatusBar::textOut(Paint &p, LPSTR lpS)
{
    if (!pOwner) return;
    getRect(p.rcPaint);
    SetBkMode(p, TRANSPARENT);
    p.rcPaint.top += drawDivide(p);
    p.rcPaint.left += GetSystemMetrics(SM_CXFRAME);
    TextOut(p, p.rcPaint.left, p.rcPaint.top, lpS, lstrlen(lpS));
}

//
// Draw the dividing line between the status bar and the rest of the client
// area.
//
WORD
StatusBar::drawDivide(Paint &p)
{
    if (!pOwner) return 0;

    // Create pen with Windows border thickness, and select it
    WORD wThick = GetSystemMetrics(SM_CYBORDER);
    Pen pen(PS_SOLID, wThick, RGB(0,0,0));
    HPEN hPen = pen.select(p);

    // Draw the dividing line
    MoveTo(p, 0,               p.rcPaint.top);
    LineTo(p, p.rcPaint.right, p.rcPaint.top);

    // Select previous pen and return thickness
    SelectObject(p, hPen);
    return wThick;
}
