//
// WINCB.CPP    Standard callback methods for Window C++ class.
//
// Created:     Bruce Clift  11/23/91
//

#include <windows.h>
#pragma hdrstop

#include "window.hpp"
#include "handler.hpp"


///////////////////////////////////////////
// INITIALIZING THE HANDLER AND MESSAGES //
///////////////////////////////////////////

//
// If getHandler is called and the data member _pHand is NULL,
// getHandler creates a Handler object to process messages.  This
// method should be overridden in derived classes!  A one-line method
// returning a pointer to a private data member is all that is needed.
//
Handler *
Window::getHandler(void)
{
    // Return Handler pointer if already set
    if (_pHand)
        return _pHand;

    // otherwise create a new Handler object, but set a flag so it
    // gets destroyed when Window is destroyed.
    f.autoHand = TRUE;
    return new Handler;
}


/////////////////////////////////////////
// REGISTERING DEFAULT WINDOW MESSAGES //
/////////////////////////////////////////

//
// regDefMsgs registers default messages for the Window class.  If
// derived classes do not wish to use Window's default callbacks, this
// method may be overridden.  If derived classes override this method
// but still wish to use Window's default callbacks, this method
// should be called explicitly.
//
void
Window::regDefMsgs(Handler *pHand)
{
    static CB MsgCbList[] = {
        { WM_DESTROY, (CBPROC) &Window::wmDestroy },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);
}

//
// wmDestroy posts the WM_QUIT message on the application queue.
//
LRESULT
Window::wmDestroy(UINT, WPARAM, LPARAM)
{
    PostQuitMessage(0);
    return 0L;
}


/////////////////////////
// MESSAGE DISPATCHING //
/////////////////////////

//
// The dispatch method uses the message list (or command list, if the message
// is WM_COMMAND) to look up the callback routine associated with the message
// received.  If neither the command nor message lookups are successful, the
// default window procedure for this object is called.
//
LRESULT
Window::dispatch(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    // For WM_CREATE, call Window's virtual setChildren method
    if (wMsg == WM_CREATE) {
        if (!setChildren())
            fail(TRUE);
    }

    // For WM_COMMAND, first look for id in command list
    CBPROC *pCB;
    Handler *pHand = getHandler();
    if (wMsg == WM_COMMAND) {
        pCB = (CBPROC *) pHand->getCmdCB(wParam);
        if (pCB)
            return (this->**pCB) (wMsg, wParam, lParam);
    }

    // For other messages (and unsuccessful WM_COMMAND's), look in message list
    pCB = (CBPROC *) pHand->getMsgCB(wMsg);
    if (pCB)
        return (this->**pCB) (wMsg, wParam, lParam);

    // Not in either list: call default window proc
    return callDefProc(wMsg, wParam, lParam);
}


//
// Call the default window procedure.  If the default procedure is not
// given, it is obtained from the Handler.  In either case its value
// is tested for being NULL before it is called.
//
LRESULT
Window::callDefProc(UINT wMsg, WPARAM wParam, LPARAM lParam)
{ return (callDefProc(getHandler()->getDefWinProc(), wMsg, wParam, lParam)); }

LRESULT
Window::callDefProc(FARPROC defWinProc, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    return (defWinProc == NULL ? 0L :
        CallWindowProc(defWinProc, _hw, wMsg, wParam, lParam));
}

