//
// WINDOW.CPP   Methods for Window C++ class.
//
// Created:     Bruce Clift  7/25/91
//

#include <windows.h>
#pragma hdrstop

#include "window.hpp"
#include "list.hpp"
#include "regwincl.hpp"


/////////////////////////
// STATIC DATA MEMBERS //
/////////////////////////

Window *    Window::pNewWin;    // Pointer to new Window object
List        Window::hwndList;   // List of current window handles


////////////////////////////////////////////
// INITIALIZATION AND DESTRUCTION METHODS //
////////////////////////////////////////////

//
// Default constructor simply clears instance variables.
//
Window::Window()
    : _hw(NULL), pCapt(NULL), pMenu(NULL), _pHand(NULL)
{}

//
// Virtual destructor calls the virtual destroy method.
//
Window::~Window()
{
    destroy();
}


//
// regClass registers a window class with Windows.
// A Handler object is first obtained to process messages for windows
// in the class, and this Window's default message reponse methods
// are registered.  Next a RegWinCls object (derived from the Windows
// WNDCLASS struct) is created and the fields filled.  Finally the
// window class is registered.
// 
BOOL
Window::regClass(HINSTANCE hInst, char *szName)
{
    // Register this Window class's default messages
    Handler *pHand = getHandler();
    setHandler(pHand);
    regDefMsgs(pHand);

    // Create RegWinCls object to register main window class
    RegWinCls rwc(szName, hInst, pHand, CS_VREDRAW | CS_HREDRAW);
    if (pMenu) rwc.menuName(pMenu); // Add menu name if specified
    setClassReg(rwc);               // Let derived class make adjustments
    return (rwc.regClass() && regChildClass(hInst));
}

//
// The virtual setClassReg method is the derived class's opportunity
// to make changes to the window class registration structure.
//
void
Window::setClassReg(RegWinCls &)
{
}

//
// The virtual regChildClass method is a hook for the derived class
// to register any child classes.
//
BOOL
Window::regChildClass(HINSTANCE)
{
    return TRUE;
}

//
// display creates and displays a window.
//
BOOL
Window::display(HINSTANCE hInst, char *szApp, char *szCapt, int nCmdShow)
{
    // Get caption name first from parameter, then from data member
    // use application name as a last resort.
    if (szCapt == NULL)
        szCapt = (pCapt ? pCapt : szApp);

    // Create and display the main window
    if (!create(szApp, szCapt, hInst))
            return FALSE;

    show(nCmdShow);
    update();
    return TRUE;
}

//
// The virtual getStyle method may be overridden to supply the display
// method with a window's style.
//
long
Window::getStyle(void)
{
    return WS_OVERLAPPEDWINDOW;
}


//
// The create method allows the user to set creation parameters as
// needed.  A pointer to this Window object is placed in a temporary
// member of the Window class.  The Handler object processing messages
// will find the pointer and add it to the window handle list (thus
// making it available for all messages, even the initial window
// messages like WM_CREATE).  If the failure flag was set during
// creation, the window is destroyed and this method fails.
//
HWND
Window::create(LPCSTR cls, LPCSTR name, HINSTANCE hInst,
    long lStyle, HMENU hMenu, HWND hwParent,
    int x, int y, int cx, int cy, LPSTR lpParam, DWORD dwExStyle)
{
    if (!cls)
        return NULL;

    // Destroy window if already created
    if (_hw) destroy();

    // If DEF_SETSTYLE is set, get the style from the virtual method
    if (lStyle == DEF_SETSTYLE)
        lStyle = getStyle();

    // Save ptr to this object in Window static data member
    pNewWin = this;

    // Create the window
    f.fail = FALSE;
    hwnd(CreateWindowEx(dwExStyle, cls, name, lStyle,
        x, y, cx, cy, hwParent, hMenu, hInst, lpParam));
    f.created = TRUE;

    // If the failure flag was set during creation,
    // destroy the window now and return failure.
    if (f.fail) {
        destroy();
        f.fail = FALSE;
        return NULL;
    }

    // For registered windows using winProcReg, or for dialog boxes 
    // and subclassed windows using winProcStd, the window will have 
    // been added to the window handle list in that window procedure. 
    // However, for other windows created using this method, the 
    // window and its handle must be added to the list here.
    newWinHnd(hwnd());

    return hwnd();
}

//
// This virtual method asks the class for its class name.
// Derived classes may override this and getClassName to
// fill in default parameters for their window class.
//
HWND
Window::create(LPCSTR name, HINSTANCE hInst, long lStyle,
    HMENU hMenu, HWND hwParent, int x, int y, int cx, int cy,
    LPSTR lpParam, DWORD dwExStyle)
{
    return create(getClassName(), name, hInst, lStyle,
        hMenu, hwParent, x, y, cx, cy, lpParam, dwExStyle);
}

LPCSTR
Window::getClassName(void)
{
    return NULL;
}

//
// For windows that were created (as opposed to existing windows which
// were found), they are destroyed and removed from our global window
// handle list.  If a Handler object had been automatically created
// (in Window::getHandler), it is deleted.
//
BOOL
Window::destroy(void)
{
    if (!_hw)
        return FALSE;

    if (f.created) {                // If window was created (not found)
        DestroyWindow(_hw);     // destroy it with Windows
        destroyWinHnd();        // and remove it from our handle list
        f.created = FALSE;
    }

    if (f.autoHand) {           // If Handler was auto-created
        delete _pHand;          // destruct it now
        _pHand = NULL;
        f.autoHand = FALSE;
    }

    _hw = NULL;
    return TRUE;                // Return success
}


///////////////////////////////////
// WINDOW HANDLE LIST MANAGEMENT //
///////////////////////////////////

//
// newWinHnd associates the given window handle with the new Window
// object pointer saved in the static data member pNewWin.
//  INPUT: hw is the window handle.
//  RETURNS: a pointer to the Window object, or NULL for failure.
//
Window *
Window::newWinHnd(HWND hw)
{
    // Get the Window object pointer from Window's static data member,
    // pNewWin (which must normally be NULL).
    if (!pNewWin) return NULL;
    Window *pWin = pNewWin;
    pNewWin = NULL;

    // Set Window's hwnd data member,
    // and add window handle & object to list
    return pWin->addWinHnd(hw);
}

//
// addWinHnd adds a Window object pointer and its window handle to the
// created window list.
//  INPUT: hw is the window handle.
//  RETURNS: a pointer to the Window object, or NULL for failure.
//
Window *
Window::addWinHnd(HWND hw)
{
    if (!hwnd(hw)) return NULL;

    // Convert object pointer to DWORD,
    // but ignore segment for near pointers
    DWORD dwNewWin;
    if (sizeof(int *) == 2)
            dwNewWin = (DWORD) (WORD) this;
    else    dwNewWin = (DWORD) this;

    if (hwndList.insert(hw, dwNewWin))
        return this;
    return NULL;
}

//
// Find a Window object given its Windows handle.
//
Window *
Window::findWinHnd(HWND hw)
{
    return ((Window *) (void FAR *) hwndList.item(hw));
}


//////////////////////////////////
// SUPPLEMENTARY OBJECT SUPPORT //
//////////////////////////////////

//
// The following virtual methods should be overridden to provide pointers
// to supplementary objects.
//
Ribbon *
Window::getRibbon(void)
{ return NULL; }

StatusBar *
Window::getStatusBar(void)
{ return NULL; }

HACCEL
Window::getAccel(void)
{ return NULL; }

Window *
Window::getActiveChild(BOOL *)
{ return NULL; }


//
// Override this method to initialize a Status Bar, Ribbon, etc.
// Returning FALSE from this routine calls the parent window's creation
// to fail.
//
BOOL
Window::setChildren(void)
{ return TRUE; }


//
// Override these methods to provide status bar help index values.
//
int
Window::getHelpIndex(void)
{ return 0; }

int
Window::getMenuHelpIndex(int)
{ return 0; }

int
Window::getSysMenuHelpIndex(int)
{ return 0; }

int
Window::getPopupHelpIndex(int)
{ return 0; }


//
// Convert the saved status bar help index into a help string.
//
char *
Window::getHelp(int)
{ return NULL; }

char *
Window::getHelp(void)
{ return NULL; }
