//
// WINSB.CPP    Methods useful for Windows using status bars.
//
// Created:     Bruce Clift  12/10/91
//
// Many of the MDI classes' features were inspired by Jeffrey Richter's
// Windows 3: A Developer's Guide.
//

#include <windows.h>
#pragma hdrstop

#include "window.hpp"
#include "statusba.hpp"
#include "paint.hpp"
#include "menu.hpp"


////////////////////////////////////////
// WINDOW STATUS BAR CALLBACK METHODS //
////////////////////////////////////////

//
// If the Window object owning the status bar DOESN'T process WM_PAINT
// (as it often will) the status bar will be painted here.  If the owner
// DOES process WM_PAINT, the paintHelp method must be called from its
// wmPaint method instead.
//
LRESULT
Window::wmPaint(UINT, WPARAM, LPARAM)
{
    // Make sure rect will be validated
    Paint p(hwnd());

    // Get pointer to status bar and paint help
    StatusBar *pSB = getStatusBar();
    if (pSB && *pSB) pSB->wmPaint(p);
    return 0L;
}

//
// WM_MENUSELECT is received by a window when the user is scrolling through
// its menu.  MDI frames also receive it for a maximized child's items.
// This method figures out where the menu item is, and obtains the 
// appropriate help index.
//
LRESULT
Window::wmMenuSelect(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    // Get pointer to status bar
    StatusBar *pSB = getStatusBar();
    if (!pSB || !*pSB) return 0L;

    // Has user stopped using menus?
    if (lParam == MAKELONG(-1, 0)) {
        pSB->setMenuHelpIndex(NULL, 0);
        return 0L;
    }

    // Get active MDI child info
    BOOL bMaximized;
    Window *pActive = getActiveChild(&bMaximized);

    // Get type of menu
    int iHelp = 0;
    switch (LOWORD(lParam) & (MF_POPUP | MF_SYSMENU)) {
    case 0:                         // Menu item NOT on app's system menu
        if (pActive) {
            if (bMaximized) {       // check if item is from child's sys menu
                WORD w = GetSubMenu(GetMenu(hwnd()), 0);
                if ((int) GetMenuState(w, wParam, MF_BYCOMMAND) != -1)
                    lParam |= MF_SYSMENU;
            }
            // Pass message on to child window
            return pActive->sendMessage(wMsg, wParam, lParam);
        }
        iHelp = pSB->getMenuHelpIndex(this, wParam);
        break;

    case MF_POPUP:                  // wParam is handle to popup menu
    {   // set scope block for w to avoid MSC error C2360:
        // 'initialization of w is skipped by case label'
        if (pActive) {
            if (bMaximized) {
                if (wParam == GetSubMenu(GetMenu(hwnd()), 0))
                    lParam |= MF_SYSMENU;
            }
            // Pass message on to child window
            return pActive->sendMessage(wMsg, wParam, lParam);
        }

        // Calculate index of top-level menu
        Menu m(hwnd());
        WORD w = m.getItemCount();
        while (w--)
            if (GetSubMenu(m, w) == (HMENU) wParam)
                break;
        // Jump over system menu
        iHelp = pSB->getPopupHelpIndex(this, w + 1);
        break;
    }

    case MF_SYSMENU :                // Frame's system menu item
        iHelp = pSB->getSysMenuHelpIndex(this, (wParam & 0x0fff) >> 4);
        break;

    case MF_POPUP | MF_SYSMENU:     // wParam is handle to app's system menu
        iHelp = pSB->getPopupHelpIndex(this, 0);
        break;
    }

    // Save the menu help index for later use
    if (iHelp)
        pSB->setMenuHelpIndex(this, iHelp);
    return 0L;
}

//
// Called when the user has temporarily stopped scrolling through a menu.
// If there is a status bar and menu help available, it is displayed.
//
LRESULT
Window::wmEnterIdle(UINT, WPARAM wParam, LPARAM)
{
    if (wParam != MSGF_MENU)    return 0L;  // Not in a menu

    // Get pointer to status bar and paint help
    StatusBar *pSB = getStatusBar();
    if (pSB && *pSB) pSB->wmEnterIdle();
    return 0L;
}

