//
// WPREG.CPP    winProcReg handles messages for registered window classes.
//
// Created:     Bruce Clift  9/18/91
//

#include <windows.h>
#pragma hdrstop

#include "handler.hpp"
#include "wincreat.hpp"
#include "window.hpp"
#include "dialog.hpp"


//
// setNewHand is called when registering a window class.  A pointer to this
// Handler object is saved in a static data member for winProcReg to find. 
// Then a temporary window is created.  When winProcReg is called by Windows to
// process the window's first message, it will see the object pointer in the
// static member and save that pointer in the window class' extra bytes.
//  INPUT: lpClass is the class name being registered.  hInst is the
// application's instance.
//
void
Handler::setNewHand(LPCSTR lpClass, HINSTANCE hInst)
{
    pNewHandler = this;                 // Save pointer to this new handler
    WinCreate wc(lpClass, NULL, hInst); // Set up window creation params
    DestroyWindow(wc.create());         // Create, then destroy window
}

//
// setPointer saves a Handler object pointer in the class extra bytes of a
// window class registered by RegWinCls.
//  INPUT: hwnd is a window in the class.  pHand is a Handler object pointer. 
//
void
Handler::setPointer(HWND hwnd)
{
    // Get index of pointer (at end of class extra bytes)
    int nIndex = GetClassWord(hwnd, GCW_CBCLSEXTRA) - sizeof(int *);

    // Save pointer (determine size at compile time if possible)
#if defined(__SMALL__) || defined(__MEDIUM__)
    SetClassWord(hwnd, nIndex, (WORD) this);
#elif defined(__LARGE__) || defined(__COMPACT__)
    SetClassLong(hwnd, nIndex, (LONG) this);
#else
    if (sizeof(int *) == 2)
            SetClassWord(hwnd, nIndex, (WORD) this);
    else    SetClassLong(hwnd, nIndex, (LONG) this);
#endif
}

//
// getPointer retrieves a Handler object pointer from the class extra bytes of
// a window class registered by RegWinCls.
//  INPUT: hwnd is a window in the class.
//  RETURNS: a Handler object pointer.
//
Handler *
Handler::getPointer(HWND hwnd)
{
    // Get index of pointer (at end of class extra bytes)
    int nIndex = GetClassWord(hwnd, GCW_CBCLSEXTRA) - sizeof(int *);

    // Get pointer (determine size at compile time if possible)
    Handler *pHand;
#if defined(__SMALL__) || defined(__MEDIUM__)
    pHand = (Handler *) GetClassWord(hwnd, nIndex);
#elif defined(__LARGE__) || defined(__COMPACT__)
    pHand = (Handler *) GetClassLong(hwnd, nIndex);
#else
    if (sizeof(int *) == 2)
            pHand = (Handler *) GetClassWord(hwnd, nIndex);
    else    pHand = (Handler *) GetClassLong(hwnd, nIndex);
#endif
    return pHand;
}

//
// winProcReg is the exported window procedure which Windows calls for windows
// whose classes were registered using the RegWinCls C++ class.  For brand new
// window classes the static data member pNewHandler is non-NULL; this function
// has been called during the creation of a temporary window.  In this case a
// pointer to the Handler object in pNewHandler is saved in the window class'
// extra bytes.  Usually, this pointer is obtained from the class extra bytes. 
// This function then gets a pointer to the Window object associated with the
// given window handle.  The dispatch method is called dispatch the message.
//
extern "C" LRESULT FAR PASCAL EXPORT
winProcReg(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    // For newly registered window classes, save Handler ptr in extra bytes
    Handler *pHand;
    if (Handler::pNewHandler) {         // If this is the temporary window
        pHand = Handler::pNewHandler;   // get pointer from static data member
        Handler::pNewHandler = NULL;    // flag that this handler isn't new now
        pHand->setPointer(hwnd);        // then save ptr in class extra bytes
        pHand->setClsExtraBytes(hwnd);  // set superclass info in extra bytes
        // No need for more processing for this temporary window
        SetWindowLong(hwnd, GWL_WNDPROC, (LONG) DefWindowProc);
        return 0L;
    }
    // Usually, get the Handler pointer from class extra bytes
    else pHand = Handler::getPointer(hwnd);
    if (!pHand)
        return CallWindowProc((FARPROC) DefWindowProc,
            hwnd, wMsg, wParam, lParam);

    // Get pointer to Window object associated with this window handle
    Window *pWin;                           // New dialog box control?
    if (Dialog::newCtls() && (wMsg==WM_NCCREATE)) {
        pHand->getClsExtraBytes(hwnd);      // Get any superclass info
        pWin = Dialog::newControl(hwnd);    // Add new dialog control
        pWin->setHandler(pHand);            // Save Handler ptr
    }
    else if (Window::pNewWin &&             // New window?
            (wMsg==WM_GETMINMAXINFO || wMsg==WM_NCCREATE)) {
        pHand->getClsExtraBytes(hwnd);      // Get any superclass info
        pWin = Window::newWinHnd(hwnd);     // Add new windows to list
        pWin->setHandler(pHand);            // Save Handler ptr
    }
    else {                                  // Must be old window
        pWin = Window::findWinHnd(hwnd);    // Get old windows from list
    }
    if (!pWin)
        return CallWindowProc((FARPROC) DefWindowProc,
            hwnd, wMsg, wParam, lParam);

    return pWin->dispatch(wMsg, wParam, lParam);
}
